#include <opencv2/core.hpp>
#include <opencv2/core/opengl.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui.hpp>
 
#include <iostream>

#ifdef __APPLE__
    #include <OpenGL/gl.h>
    #include <OpenGL/gl3.h>
    #include <GLUT/glut.h>
#else
    #include <GL/glew.h>
    #include <GL/glut.h>
#endif

using namespace cv;
using namespace std;

Mat image;
GLuint imageTexture;
 
void displayMe(void)
{
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    glTexImage2D(       GL_TEXTURE_2D,       // Type of texture
                        0,                   // Pyramid level (for mip-mapping) - 0 is the top level
                        GL_RGB,              // Internal colour format to convert to
                        image.cols,          // Image width  i.e. 640 for Kinect in standard mode
                        image.rows,          // Image height i.e. 480 for Kinect in standard mode
                        0,                   // Border width in pixels (can either be 1 or 0)
                        GL_RGB,              // Input image format (i.e. GL_RGB, GL_RGBA, GL_BGR etc.)
                        GL_UNSIGNED_BYTE,    // Image data type
                        image.ptr());        // The actual image data itself
    glEnd();
    glFlush();
}
 
int main(int argc, char** argv)
{
    string image_path = samples::findFile("chart.png");
    Mat image = imread(image_path, IMREAD_COLOR);

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    glGenTextures(1, &imageTexture);
    glBindTexture(GL_TEXTURE_2D, imageTexture);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    // Set texture clamping method
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

#if (CV_VERSION_MAJOR >= 4)
    cvtColor(image, image, cv::COLOR_BGR2RGB);
#else
    cvtColor(image, image, CV_BGR2RGB);
#endif

    //GLint dims[4] = {0};
    //glGetIntegerv(GL_VIEWPORT, dims);
    //GLint fbWidth = dims[2];
    //GLint fbHeight = dims[3];
    //cout << fbWidth << "," << fbHeight << endl;
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_SINGLE);
    glutInitWindowSize(1920, 1080);
    glutInitWindowPosition(0, 0);
    glutCreateWindow("Hello world!");
    glutFullScreen();

    glutDisplayFunc(displayMe);
    glutMainLoop();
    return 0;
}